var assert = require("chai").assert;
var events = require("events");
var _ = require("lodash");
var techPerfLogger = require("../techPerfLogger.js");
var techRuuid = require("node-tech-ruuid");
var logger = require("node-tech-logger");
logger.setup({});

describe("techPerfLogger", function() {

    var db, http, ruuid = null;

    beforeEach(function() {
        db = new events.EventEmitter();
        http = new events.EventEmitter();
        ruuid = techRuuid.create();
    });


    it("Accumulates durations and count for various requests type", function() {

        var perfLogger = techPerfLogger({
            reporters: [],
            owner: "av-accounts"
        });

        perfLogger.addListener(http, "http", function(event) {
            return "http" + event.category;
        });

        perfLogger.addListener(db, "sql", function(event) {
            return "sql" + event.category;
        });

        perfLogger.startRequest(ruuid, "/foo/bar", "GET");

        db.emit("sql", {
            ruuid: ruuid,
            type: "sql",
            category: "Read",
            duration: {
                ms: 1500
            }
        });

        db.emit("sql", {
            ruuid: ruuid,
            type: "sql",
            category: "read",
            duration: {
                ms: 1000
            }
        });

        db.emit("sql", {
            ruuid: ruuid,
            type: "sql",
            category: "create",
            duration: {
                ms: 300
            }
        });

        http.emit("http", {
            ruuid: ruuid,
            type: "http",
            category: "core",
            duration: {
                ms: 200
            }
        });

        var fact = perfLogger.gatherFact(ruuid, {
            ms: 5000
        }, "finish");

        assert.deepEqual(fact, {
            attributes : [
                { name : "tx", value : ruuid },
                { name : "duration", value : 5000},
                { name : "nb_sql_read", value : 2},
                { name : "dur_sql_read", value : 2500},
                { name : "nb_sql_create", value : 1},
                { name : "dur_sql_create", value : 300},
                { name : "nb_http_core", value : 1},
                { name : "dur_http_core", value : 200},
            ]
        });

    });

    it("schedules posting for later", function() {

        var called = false;
        var reporters = [{
            reportFact: function(fact) {
                var duration = _.find(fact.attributes, function (attribute) {
                    return attribute.name === "duration";
                });
                assert.ok(duration);
                assert.equal(duration.value, 5000);

                called = true;
            }
        }];

        var perfLogger = techPerfLogger({
            reporters: reporters,
            owner: "av-accounts"
        });

        perfLogger.startRequest(ruuid, "/foo/bar", "GET");

        perfLogger.endRequest(ruuid, {
            ms: 5000
        });

        assert.ok(called, "AvData should have been called");

    });

});
